@@ -1,12 +1,13 @@
-# Copyright 2021-2023 Teemu Ikonen
+# Copyright 2021-2024 Teemu Ikonen
 # SPDX-License-Identifier: GPL-3.0-only
 
 import sys
-from .application import SatelliteApp
+
+from .application import Application
 
 
 def main():
-    app = SatelliteApp()
+    app = Application()
     app.run()
     sys.exit(0)
 
