@@ -1,4 +1,4 @@
-# Copyright 2021-2024 Teemu Ikonen
+# Copyright 2021-2025 Teemu Ikonen
 # SPDX-License-Identifier: GPL-3.0-only
 
 import argparse
@@ -18,12 +18,14 @@ from .gpx import GpxSaver
 from .mm_glib_source import ModemManagerGLibNmeaSource
 from .nmeasource import (
     GnssShareNmeaSource,
+    GpsShareNmeaSource,
     GpsdNmeaSource,
     ModemError,
     ModemLockedError,
     ModemNoNMEAError,
     NmeaSourceNotFoundError,
     ReplayNmeaSource,
+    UnixSocketNmeaSource,
 )
 from .util import bearing_to_arrow, now, show_alertdialog, unique_filename
 from .window import Window
@@ -60,15 +62,15 @@ class Application(Adw.Application):
             help='Output satellite data to console')
         parser.add_argument(
             '-s', '--source', dest='source',
-            choices=['auto', 'gnss-share', 'gpsd', 'mm', 'replay'],
             default='auto',
             help="Select NMEA source. Options are:\n"
                  "'auto' (default) Automatic source detection\n"
                  "'gnss-share' Read from gnss-share socket\n"
+                 "'gps-share' Read from gps-share socket\n"
                  "'gpsd' Read from local gpsd (experimental)\n"
                  "'mm' ModemManager GPS\n"
-                 "'replay' Replay NMEAs from a text file (for testing)\n")
-        parser.add_argument('nmeafile', nargs='?')
+                 "'socket=<socketfile>' Read NMEAs from a unix socket\n"
+                 "'replay=<nmeafile>' Replay NMEAs from a text file (for testing)\n")
         parser.add_argument(
             '-q', '--quirks', dest='quirks', nargs='*',
             choices=['auto', 'quectel-talker', 'no-agps'],
@@ -89,7 +91,7 @@ class Application(Adw.Application):
 
         # Internal state
         self.last_mode = 1
-        self.last_data = {'visibles': [], 'actives': []}
+        self.last_data = None
         self.last_speed = None
         self.last_update = None
         self.had_error = False
@@ -190,7 +192,7 @@ class Application(Adw.Application):
             website="https://codeberg.org/tpikonen/satellite/",
             comments="A program for showing navigation satellite data",
             license_type=Gtk.License.GPL_3_0_ONLY,
-            copyright="Copyright 2021-2024 Teemu Ikonen",
+            copyright="Copyright 2021-2025 Teemu Ikonen",
         )
         adlg.present(self.window)
 
@@ -249,15 +251,12 @@ class Application(Adw.Application):
         self.window.toast_overlay.add_toast(toast)
 
     def on_copy_coordinates(self, *args):
-        if self.last_data is not None:
-            lat = self.last_data.get('latitude')
-            lon = self.last_data.get('longitude')
-
-        if (self.last_data is None or (lat is None and lon is None)):
+        latlon = self.last_data.get('latlon') if self.last_data else None
+        if latlon is None:
             text = "<b>Coordinates are not known,\ncannot copy</b>"
         else:
             cb = Gdk.Display().get_default().get_clipboard()
-            ctext = f"{lat:0.6f}, {lon:0.6f}"
+            ctext = f"{latlon[0]:0.6f}, {latlon[1]:0.6f}"
             cb.set(ctext)
             text = f"<b>'{ctext}'\ncopied to clipboard</b>"
 
@@ -288,9 +287,10 @@ class Application(Adw.Application):
             return (False, dtext)
 
     def init_source(self, unused):
-        auto_sources = {
+        sources = {
             # argname: (fullname, class)
             'gnss-share': ('gnss-share', GnssShareNmeaSource),
+            'gps-share': ('gps-share', GpsShareNmeaSource),
             'gpsd': ('gpsd', GpsdNmeaSource),
             'mm': ('ModemManager', ModemManagerGLibNmeaSource),
         }
@@ -298,7 +298,7 @@ class Application(Adw.Application):
         if self.args.source == 'auto':
             self.log_msg("Detecting NMEA sources...")
             ok = False
-            for source in auto_sources.values():
+            for source in sources.values():
                 ok, _dtext = self.try_to_init_source(source, autodetect=True)
                 if ok:
                     break
@@ -311,19 +311,38 @@ class Application(Adw.Application):
                 return GLib.SOURCE_REMOVE
         else:
             self.log_msg(f'NMEA source "{self.args.source}" selected')
-            if self.args.source == 'replay':
+            if self.args.source.startswith('replay='):
+                nmeafile = self.args.source.split('=')[1].strip('\'" ')
                 try:
                     self.source = ReplayNmeaSource(
                         self.location_update_cb,
                         refresh_rate=self.refresh_rate,
-                        replay_filename=self.args.nmeafile,
+                        replay_filename=nmeafile,
                         quirks=self.args.quirks)
                     self.source.initialize()
                     ok, dtext = True, ""
                 except NmeaSourceNotFoundError as e:
                     ok, dtext = False, str(e)
+            elif self.args.source.startswith('socket='):
+                socketfile = self.args.source.split('=')[1].strip('\'" ')
+                try:
+                    self.source = UnixSocketNmeaSource(
+                        self.location_update_cb,
+                        refresh_rate=self.refresh_rate,
+                        socket_file_path=socketfile,
+                        quirks=self.args.quirks)
+                    self.source.initialize()
+                    ok, dtext = True, ""
+                except FileNotFoundError as e:
+                    ok, dtext = False, str(e)
             else:
-                ok, dtext = self.try_to_init_source(auto_sources[self.args.source])
+                if self.args.source not in sources:
+                    self.log_msg(f"Unknown NMEA source '{self.args.source}'")
+                    show_alertdialog(
+                        self.window, "Error initializing NMEA source",
+                        f"Unknown NMEA source '{self.args.source}'", "OK")
+                    return GLib.SOURCE_REMOVE
+                ok, dtext = self.try_to_init_source(sources[self.args.source])
 
             if not ok:
                 self.log_msg(f"Error initializing NMEA source: {dtext}")
@@ -479,6 +498,7 @@ class Application(Adw.Application):
             show_dialog = False
             etext = str(e)
             dtext = None
+
             if isinstance(e, ModemLockedError):
                 dtext = "Please unlock the Modem"
                 show_dialog = True
@@ -493,11 +513,13 @@ class Application(Adw.Application):
                     self.window.sensitive(False)
             else:
                 dtext = etext if etext else "Unknown error"
+
             if not self.had_error:
                 if show_dialog:
                     show_alertdialog(self.window, "Error", dtext, "OK")
                 elif dtext is not None:
                     self.log_msg(dtext)
+
             self.had_error = True
             if self.last_data is None:
                 return
