@@ -1,4 +1,4 @@
-# Copyright 2021-2024 Teemu Ikonen
+# Copyright 2021-2025 Teemu Ikonen
 # SPDX-License-Identifier: GPL-3.0-only
 
 import json
@@ -130,6 +130,14 @@ class GnssShareNmeaSource(UnixSocketNmeaSource):
         self.type = "gnss-share"
 
 
+class GpsShareNmeaSource(UnixSocketNmeaSource):
+    def __init__(self, update_callback, **kwargs):
+        super().__init__(update_callback,
+                         socket_file_path='/var/run/gps-share.sock',
+                         **kwargs)
+        self.type = "gps-share"
+
+
 class InetSocketNmeaSource(SocketNmeaSource):
     def __init__(self,
                  update_callback,
